"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _trieCodec = _interopRequireDefault(require("@polkadot/trie-codec"));

var _util = require("@polkadot/util");

var _Cache = _interopRequireDefault(require("./Cache"));

var _Impl = _interopRequireDefault(require("./Impl"));

var _defaults = _interopRequireDefault(require("./defaults"));

var _util2 = require("./util");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const l = (0, _util.logger)('db/struct');
const TRIE_BRANCH_LEN = 17;

class FileStructDb extends _Impl.default {
  constructor() {
    super(...arguments);
    this._cache = new _Cache.default(16 * 1024);
  }

  drop() {
    l.error('drop() is not implemented');
  }

  empty() {
    l.error('empty() is not implemented');
  }

  rename(base, file) {
    l.error('rename() is not implemented');
  }

  size() {
    l.error('size() is not implemented');
    return 0;
  }

  maintain(fn) {
    fn({
      isCompleted: true,
      keys: 0,
      percent: 100
    });
  }

  del(key) {
    l.error('del() is not implemented');
  }

  _get(key) {
    const keyInfo = this._findValue((0, _util2.serializeKey)(key));

    if (!keyInfo || !keyInfo.valData) {
      return null;
    } else if (!this._isTrie) {
      return keyInfo.valData;
    } else if (!keyInfo.valData[0]) {
      return keyInfo.valData.subarray(1);
    }

    const recoded = [];
    let offset = 1;

    try {
      while (offset < keyInfo.valData.length) {
        if (keyInfo.valData[offset]) {
          const [length, keyAt] = (0, _util.compactFromU8a)(keyInfo.valData.subarray(offset));

          const keyBuff = this._readKey(0, keyAt.toNumber());

          recoded.push(keyBuff.subarray(0, _defaults.default.KEY_DATA_SIZE));
          offset += length;
        } else {
          recoded.push(null);
          offset += 1;
        }
      }
    } catch (error) {
      console.error(error.message, keyInfo.valData);
      throw error;
    }

    return _trieCodec.default.encode(recoded);
  }

  get(key) {
    // l.debug(() => ['get', { key }]);
    const keyStr = key.toString();

    const cached = this._cache.get(keyStr);

    if (cached) {
      return cached;
    }

    const value = this._get(key);

    if (value) {
      this._cache.set(keyStr, value);
    }

    return value;
  }

  put(key, value) {
    // l.debug(() => ['put', { key, value }]);
    let adjusted;

    if (this._isTrie) {
      const decoded = _trieCodec.default.decode(value);

      const isEncodable = Array.isArray(decoded) && decoded.length === TRIE_BRANCH_LEN && !decoded.some(value => value && value.length !== 32); // extension nodes are going away anyway, so just ignore, no worse off.

      if (isEncodable) {
        const recoded = [TRIE_BRANCH_LEN];

        for (let index = 0; index < TRIE_BRANCH_LEN; index++) {
          const u8a = decoded[index];

          if (u8a) {
            const entry = (0, _util2.serializeKey)(u8a);

            const keyInfo = this._findValue(entry, null, false);

            recoded.push(...(0, _util.compactToU8a)(keyInfo.keyAt));
          } else {
            recoded.push(0);
          }
        }

        adjusted = new Uint8Array(recoded);
      } else {
        adjusted = new Uint8Array(value.length + 1);
        adjusted.set(value, 1);
      }
    }

    this._cache.set(key.toString(), value);

    this._findValue((0, _util2.serializeKey)(key), adjusted || value);
  }

}

exports.default = FileStructDb;